@@ -24,17 +24,19 @@
 //!
 //! ## Parallel Processing
 //!
-//! - [`stream.par_map(config, fn)`](ParStreamExt::par_map) processes stream items in parallel closures.
+//! - [`stream.par_map(config, map_fn)`](ParStreamExt::par_map) processes stream items in parallel closures.
 //! - [`stream.par_then(config, fut)`](ParStreamExt::par_then) processes stream items in parallel futures.
 //! - [`par_map_unordered()`](ParStreamExt::par_map_unordered) and [`par_then_unordered()`](ParStreamExt::par_then_unordered)
 //!   are unordered variances.
-//! - [`try_par_map()`](TryParStreamExt::try_par_map), [`try_par_then()`](TryParStreamExt::try_par_then),
-//!   [`try_par_then_unordered()`](TryParStreamExt::try_par_then_unordered) are the fallible variances.
+//! - [`try_par_map()`](FallibleParStreamExt::try_par_map), [`try_par_then()`](FallibleParStreamExt::try_par_then),
+//!   [`try_par_then_unordered()`](FallibleParStreamExt::try_par_then_unordered) are the fallible variances.
 //!
 //! ## Distributing Patterns
 //!
-//! - [`stream.tee(buf_size)`](ParStreamExt::tee) creates a copy of a stream.
-//! - [`stream.scatter(buf_size)`](ParStreamExt::scatter) forks a stream into parts.
+//! - [`stream.broadcast(buf_size)`](ParStreamExt::broadcast) broadcasts copies of elements to multiple receivers.
+//! - [`stream.tee(buf_size)`](ParStreamExt::tee) creates copies the stream at any time.
+//!   Unlike [`broadcast()`](ParStreamExt::broadcast), receivers can start consuming at any time.
+//! - [`stream.scatter(buf_size)`](ParStreamExt::scatter) sends each element to one of existing receivers.
 //! - [`gather(buf_size, streams)`](gather) merges multiple streams into one stream.
 //!
 //! ### Scatter-Gather Pattern
@@ -79,27 +81,34 @@
 //! # }
 //! ```
 //!
-//! ### Tee-Zip Pattern
+//! ### Broadcast-Zip Pattern
 //!
-//! Another example is to construct a tee-zip pattern that clones each element to
+//! Another example is to construct a broadcast-zip pattern that clones each element to
 //! several concurrent workers, and pairs up outputs from each worker.
 //!
 //! ```rust
-//! # use futures::stream::StreamExt;
-//! # use par_stream::ParStreamExt;
+//! # use futures::prelude::*;
+//! # use par_stream::prelude::*;
 //!
 //! async fn main_async() {
-//!     let orig: Vec<_> = (0..1000).collect();
+//!     let data = vec![2, -1, 3, 5];
 //!
-//!     let rx1 = futures::stream::iter(orig.clone()).tee(1);
-//!     let rx2 = rx1.clone();
-//!     let rx3 = rx1.clone();
+//!     let mut guard = futures::stream::iter(data.clone()).broadcast(3);
+//!     let rx1 = guard.register();
+//!     let rx2 = guard.register();
+//!     let rx3 = guard.register();
+//!     guard.finish(); // the guard is dropped so that registered streams can start
 //!
-//!     let fut1 = rx1.map(|val| val).collect();
-//!     let fut2 = rx2.map(|val| val * 2).collect();
-//!     let fut3 = rx3.map(|val| val * 3).collect();
+//!     let join = rx1
+//!         .map(|v| v * 2)
+//!         .zip(rx2.map(|v| v * 3))
+//!         .zip(rx3.map(|v| v * 5));
 //!
-//!     let (vec1, vec2, vec3): (Vec<_>, Vec<_>, Vec<_>) = futures::join!(fut1, fut2, fut3);
+//!     let collected: Vec<_> = join.collect().await;
+//!     assert_eq!(
+//!         collected,
+//!         vec![((4, 6), 10), ((-2, -3), -5), ((6, 9), 15), ((10, 15), 25)]
+//!     );
 //! }
 //!
 //! # #[cfg(feature = "runtime-async-std")]
@@ -122,12 +131,12 @@
 //!
 //! ## Item Ordering
 //!
-//! - [`stream.wrapping_enumerate()`](ParStreamExt::wrapping_enumerate) is like [`enumerate()`](futures::StreamExt::enumerate),
+//! - [`stream.wrapping_enumerate()`](IndexedStreamExt::wrapping_enumerate) is like [`enumerate()`](futures::StreamExt::enumerate),
 //!   but wraps around to zero after reaching [usize::MAX].
-//! - [`stream.reorder_enumerated()`](ParStreamExt::reorder_enumerated) accepts a `(usize, T)` typed stream and
+//! - [`stream.reorder_enumerated()`](IndexedStreamExt::reorder_enumerated) accepts a `(usize, T)` typed stream and
 //!   reorder the items according to the index number.
-//! - [`stream.try_wrapping_enumerate()`](TryParStreamExt::try_wrapping_enumerate) and
-//!   [`stream.try_reorder_enumerated()`](TryParStreamExt::try_reorder_enumerated) are fallible counterparts.
+//! - [`stream.try_wrapping_enumerate()`](FallibleIndexedStreamExt::try_wrapping_enumerate) and
+//!   [`stream.try_reorder_enumerated()`](FallibleIndexedStreamExt::try_reorder_enumerated) are fallible counterparts.
 //!
 //! The item ordering combinators are usually combined with unordered concurrent processing methods,
 //! allowing on-demand data passing between stages.
@@ -135,18 +144,18 @@
 //! ```ignore
 //! stream
 //!     // mark items with index numbers
-//!     .wrapping_enumerate()
+//!     .enumerate()
 //!     // a series of unordered maps
-//!     .par_then_unordered(config, fn)
-//!     .par_then_unordered(config, fn)
-//!     .par_then_unordered(config, fn)
+//!     .par_then_unordered(config, map_fn)
+//!     .par_then_unordered(config, map_fn)
+//!     .par_then_unordered(config, map_fn)
 //!     // reorder the items back by indexes
 //!     .reorder_enumerated()
 //! ```
 //!
 //! ## Configure Number of Workers
 //!
-//! The `config` parameter of [`stream.par_map(config, fn)`](ParStreamExt::par_map) controls
+//! The `config` parameter of [`stream.par_map(config, map_fn)`](ParStreamExt::par_map) controls
 //! the number of concurrent workers and internal buffer size. It accepts the following values.
 //!
 //! - `None`: The number of workers defaults to the number of system processors.
@@ -159,21 +168,18 @@
 /// Commonly used traits.
 pub mod prelude {
     pub use super::{
-        slice::{Chunk, SliceExt},
-        stream::ParStreamExt,
-        try_stream::TryParStreamExt,
+        stream::{IndexedStreamExt, ParStreamExt},
+        try_stream::{FallibleIndexedStreamExt, FallibleParStreamExt},
     };
 }
 
 mod common;
 mod config;
-mod error;
 mod rt;
-mod slice;
 mod stream;
 mod try_stream;
+mod utils;
 
 pub use config::*;
-pub use slice::*;
 pub use stream::*;
 pub use try_stream::*;
